#include <iostream>
#include <iomanip>
#include <qstring.h>

#include "rurl.h"

using namespace std;
using namespace Relative;

void test(const QString &message, bool expr, const QString &testVal = QString::null)
{
    if (expr)
    {
        cout << "ok" << "      testing " << message << " ";
        if (!testVal.isEmpty())
            cout << "   test value = " << testVal;
        cout << endl;
    }
    else
    {
        cerr << "failed" << " testing " << message << " ";
        if (!testVal.isEmpty())
            cerr << "   test value = " << testVal;
        cerr << endl;
    }
}

void RName_tests()
{
    Name name = "boo/test.h";
    Name name2("boo/test.h");
    Name name3("boo/test.h/");
    Name name4("boo///test.h//");
    Name name4a("boo///test.h//", Name::File);
    Name name5("boo///data/////test.tar.h");
    Name name6("boo.dir", Name::Directory);
    Name name7("/boo.tar");
    Name name8("");
    Name name9("../../../name1/name2/");
    Name name10("../../../name1/name2.file");

    //constructor
    test("constructor #1 - is file?", name.isFile(), name.rurl()) ;
    test("constructor #2 - is directory?", name3.isDirectory(), name3.rurl());
    test("constructor #3 - no slash at the beginning of a file", !name.rurl().startsWith("/"));
    test("constructor #3 - no slash at the beginning of a dir", !name3.rurl().startsWith("/"));
    test("constructor #4 - no slash at the end of file", !name4a.rurl().endsWith("/"));
    test("constructor #5 - slash at the end of directory", name3.rurl().endsWith("/"));
    test("constructor #6 - empty rurl creation", name8.rurl() == "");
    test("constructor #7 - strange empty rurl ('/') creation", Name("/").rurl() == "");
    test("constructor #8 - strange empty rurl ('////') creation", Name("////").rurl() == "");
    test("constructor #9 - is file?", name10.isFile(), name10.rurl()) ;
    test("constructor #10 - is directory?", name9.isDirectory(), name9.rurl());
    cout << endl;
    
    //cleanRURL
    test("cleanRURL in constructor", !name4.rurl().contains("//"), name4.rurl());
    test("cleanRURL using cleanName", !Name::cleanName("boo///test.h//").contains("//"), Name::cleanName("boo///test.h//"));
    cout << endl;
    
    //operator ==() and !=()
    test("operator == with QString argument ", name == "/boo/test.h");
    test("operator == with RURL argument ", name == name2);
    test("operator == for empty rurl with QString argument ", name8 == "");
    test("operator != with RURL argument which is directory ", name != name3);
    cout << endl;
    
    //fileName(), extension(), directory()
    test("fileName (file relative name)", name.fileName() == "test.h", name.fileName());
    test("fileName (directory relative name)", name3.fileName() != "test.h", name3.fileName());
    test("fileName (for empty rurl)", name8.fileName() == "", name8.fileName());
    test("complete extension (file relative name)", name5.extension() == "tar.h", name5.extension());
    test("short extension (file relative name)", name5.extension(false) == "h", name5.extension(false));
    test("extension (directory relative name)", name3.extension() == "", name3.extension());
    test("extension (empty rurl)", name8.extension() == "", name8.extension());
    test("directory for normal file name", name5.directory() == "boo/data/", name5.directory());
    test("directory for normal directory", name4.directory() == "boo/test.h/", name4.directory());
    test("directory for short directory", name6.directory() == "boo.dir/", name6.directory());
    test("directory for file", name7.directory() == "", name7.directory());
    test("directory for empty url", name8.directory() == "", name8.directory());
    cout << endl;
    
    //valid and correctName
    test("valid", name5.isValid(), name5.rurl());
    test("correctName", Name::correctName("//test///boo////test.dir", Name::Directory) == "test/boo/test.dir/", Name::correctName("//test///boo////test.dir", Name::Directory));
    test("is directory for directory", name4.isDirectory());
    test("is file for directory", !name4.isFile());
    test("is directory for file", !name.isDirectory());
    test("is file for file", name.isFile());
    test("is directory for empty rurl", !name8.isDirectory());
    test("is file for empty rurl", name8.isFile());
    cout << endl;

    //addPath - file
    Name namex = name5;
    namex.addPath("hoho/boo2/");
    test("addPath for file (usual path)", namex == "boo/data/hoho/boo2/test.tar.h", namex.rurl());
    namex = name5;
    namex.addPath("///hoho///boo2///");
    test("addPath for file (strange path 1)", namex == "boo/data/hoho/boo2/test.tar.h", namex.rurl());
    namex = name5;
    namex.addPath("///hoho///boo2");
    test("addPath for file (strange path 2)", namex == "/boo/data/hoho/boo2/test.tar.h", namex.rurl());
    cout << endl;

    //addPath - directory
    namex = name4;
    namex.addPath("hoho/boo2/");
    test("addPath for directory (usual path)", namex == "boo/test.h/hoho/boo2/", namex.rurl());
    namex = name4;
    namex.addPath("///hoho///boo2///");
    test("addPath for directory (strange path 1)", namex == "boo/test.h/hoho/boo2/", namex.rurl());
    namex = name4;
    namex.addPath("///hoho///boo2");
    test("addPath for directory (strange path 2)", namex == "boo/test.h/hoho/boo2/", namex.rurl());
    namex = name9;
    namex.addPath("///hoho///boo2");
    test("addPath for directory with ..", namex == "../../../name1/name2/hoho/boo2/", namex.rurl());
    cout << endl;

    //relative name
    test("relativeName", Name::relativeName("/home/test/project/proj1/src", "/home/src/test.h") == "../../../../src/test.h", Name::relativeName("/home/test/project/proj1/src", "/home/src/test.h").rurl());
    test("relativeName", Name::relativeName("/home/test/project/proj1/src", "/usr/src/test.h") == "../../../../../usr/src/test.h", Name::relativeName("/home/test/project/proj1/src", "/usr/src/test.h").rurl());
    test("relativeName", Name::relativeName("/home/test/project/proj1/src", "/home/test/project/proj1/src/usr/src/test.h") == "usr/src/test.h", Name::relativeName("/home/test/project/proj1/src", "/home/test/project/proj1/src/usr/src/test.h").rurl());
    test("relativeName", Name::relativeName("/home/test/project/proj1/src/", "/home/test/project/proj1/src/usr/src/test.h") == "usr/src/test.h", Name::relativeName("/home/test/project/proj1/src/", "/home/test/project/proj1/src/usr/src/test.h").rurl());
    test("relativeName", Name::relativeName("/home/test", "/home/test/test.h") == "test.h", Name::relativeName("/home/test", "/home/test/test.h").rurl());
    cout << endl;
}

void RURL_tests()
{
    KURL kurl1; kurl1.setPath("/boo/dir/base1");
    KURL kurl2; kurl2.setPath("/boo/dir/base1/");
    KURL kurl3; kurl3.setPath("/boo/dir/base1/sub1/sub2/file.ext.gz");
    KURL kurl4; kurl4.setPath("/boo/dir/base1/sub1/");

    URL url1(kurl1, kurl3);
    URL url2(kurl2, kurl3);
    URL url3(kurl2, kurl4);
    URL url4(kurl1, "sub1/sub2/file.ext.gz", true);
    URL url5(kurl1, "sub1/", true);
    URL url6("/boo/dir/base1/", "/boo/dir2/sub2", false, Name::Directory);
    URL url7("/boo/dir/base1/", "/boo/dir2/sub2/", false);
    URL url8("/boo/dir/base1/", "/boo/dir2/sub2/test.f", false);

    test("constructor #1 - relative url to file, KURL args", url1.rurl() == "sub1/sub2/file.ext.gz", url1.rurl());
    test("constructor #2 - relative url to file, KURL args", url2.rurl() == "sub1/sub2/file.ext.gz", url2.rurl());
    test("constructor #3 - relative url to directory, KURL args", url3.rurl() == "sub1/", url3.rurl());
    test("constructor #4 - relative url to file, KURL and QString args", url4.rurl() == "sub1/sub2/file.ext.gz", url4.rurl());
    test("constructor #5 - relative url to directory, KURL and QString args", url5.rurl() == "sub1/", url5.rurl());
    test("constructor #6 - url to directory, QString and QString args, directory is not a direct subdir of base", url6.rurl() == "../../dir2/sub2/", url6.rurl());
    test("constructor #7 - url to directory, QString and QString args, directory is not a direct subdir of base, auto detect", url7.rurl() == "../../dir2/sub2/", url7.rurl());
    test("constructor #8 - url to file, QString and QString args, file is not under a direct subdir of base, auto detect", url8.rurl() == "../../dir2/sub2/test.f", url8.rurl());
    cout << endl;

    test("base path (file url)", url8.basePath() == "/boo/dir/base1/");
    test("url (file url)", url8.url() == KURL("/boo/dir2/sub2/test.f"), url8.url().url());
    test("url path (file url)", url8.urlPath() == "/boo/dir2/sub2/test.f", url8.urlPath());
    test("url directory (file url)", url8.urlDirectory() == "/boo/dir2/sub2/", url8.urlDirectory());
    cout << endl;

    test("base path (dir url)", url7.basePath() == "/boo/dir/base1/");
    test("url (dir url), with slash in KURL to compare", url7.url() == KURL("/boo/dir2/sub2/"), url7.url().url());
    test("url (dir url), w/o slash in KURL to compare", url7.url() != KURL("/boo/dir2/sub2"), url7.url().url());
    test("url path (dir url)", url7.urlPath() == "/boo/dir2/sub2/", url7.urlPath());
    test("url directory (dir url)", url7.urlDirectory() == "/boo/dir2/sub2/", url7.urlDirectory());
    cout << endl;

    test("relativeTo (dir url)", url7.relativeTo("/").url() == KURL("/boo/dir2/sub2/"), url7.relativeTo("/").url().url());
    test("relativeTo (dir url)", url7.relativeTo("/boo").url() == KURL("/boo/dir2/sub2/"), url7.relativeTo("/boo").url().url());
    test("relativeTo (dir url)", url7.relativeTo("/boo/foo").url() == KURL("/boo/dir2/sub2/"), url7.relativeTo("/boo/foo").url().url());
    test("relativeTo (dir url)", url7.relativeTo("/foo").url() == KURL("/boo/dir2/sub2/"), url7.relativeTo("/foo").url().url());
    test("relativeTo (dir url)", url7.relativeTo("/foo/goo").url() == KURL("/boo/dir2/sub2/"), url7.relativeTo("/foo/goo").url().url());
    cout << endl;

    test("relativeTo (dir url)", url7.relativeTo("/").rurl() == "boo/dir2/sub2/", url7.relativeTo("/").rurl());
    test("relativeTo (dir url)", url7.relativeTo("/boo").rurl() == "dir2/sub2/", url7.relativeTo("/boo").rurl());
    test("relativeTo (dir url)", url7.relativeTo("/boo/foo").rurl() == "../dir2/sub2/", url7.relativeTo("/boo/foo").rurl());
    test("relativeTo (dir url)", url7.relativeTo("/foo").rurl() == "../boo/dir2/sub2/", url7.relativeTo("/foo").rurl());
    test("relativeTo (dir url)", url7.relativeTo("/foo/goo").rurl() == "../../boo/dir2/sub2/", url7.relativeTo("/foo/goo").rurl());
    cout << endl;

    test("relativeURL (two KURL args)", URL::relativeURL(kurl2, kurl3).rurl() == "sub1/sub2/file.ext.gz");
    test("relativeTo (KURL and QString relative)", URL::relativeURL(kurl2, "sub1/sub2/file.ext.gz", true).rurl() == "sub1/sub2/file.ext.gz");
    test("relativeTo (KURL and QString full)", URL::relativeURL(kurl2, "/boo/dir/base1/sub1/sub2/file.ext.gz", false).rurl() == "sub1/sub2/file.ext.gz");
    cout << endl;

    test("operator ==", url7 == url7);
    test("operator !=", url7 != url8);
    cout << endl;
}

void FileAndDirectory_tests()
{
    KURL kurl1; kurl1.setPath("/boo/dir/base1");
    KURL kurl2; kurl2.setPath("/boo/dir/base1/");
    KURL kurl3; kurl3.setPath("/boo/dir/base1/sub1/sub2/file.ext.gz");
    KURL kurl4; kurl4.setPath("/boo/dir/base1/sub1/");

    File url1(kurl1, kurl3);
    Directory url2(kurl2, kurl4);

    test("constructor #1 - relative url to file, KURL args", url1.rurl() == "sub1/sub2/file.ext.gz", url1.rurl());
    test("constructor #2 - relative url to directory, KURL args", url2.rurl() == "sub1/", url2.rurl());

    url1.setRURL("sub3/boo.tar.gz/");
    test("setRURL to File object with ending slash in rurl", url1.rurl() == "sub3/boo.tar.gz", url1.rurl());
    url2.setRURL("sub3/boo");
    test("setRURL to Directory object without ending slash in rurl", url2.rurl() == "sub3/boo/", url2.rurl());
    cout << endl;
}

int main()
{
    cout << "========== class RURL::Name ==========" << endl << endl;
    RName_tests();
    cout << endl << endl << endl << endl << "========== class RURL::URL ==========" << endl << endl;
    RURL_tests();
    cout << endl << endl << endl << endl << "========== classes File and Directory ==========" << endl << endl;
    FileAndDirectory_tests();

    return 0;
}
